/* *************************************************************************
   *  Kindermann                                  WPK Produktentwicklung   *
   *                                              Schuljahr: 2012/13       *
   *                                                                       *
   *                 Sony PS2 (Playstation) - Interface                    *
   *                                                                       *
   *  Dieses Programm  wurde fr  einen ATMEL AT90S2313  geschrieben. Es   *
   *  dient zur Umsetzung des Sony-PS2-Protokolls einer Knig  Tanzmatte   *
   *  in digitale Joystick-Signale. Damit kann die Tanzmatte am Gameport   *
   *  des Pacman-Spiels angeschlossen werden.                              *
   *                                                                       *
   *  Dieses Programm basiert auf dem  PS2-Joystick-Projekt  von  Thomas   *
   *  Scherrer (http://www.webx.dk/robot-crawler/ps-joy.htm) und kann un-  *
   *  ter den Bedingungen der GNU General Public License (GPL)  weiterge-  *
   *  ben und/oder modifiziert werden.                                     *
   *                                                                       *
   *************************************************************************/



//                AT90S2313 8MHz-Quarz
//
//                     +-\/-+
//            RESET   1|    |20   Vcc
//  CLK         PD0   2|    |19   PB7      LINKS
//  DAT         PD1   3|    |18   PB6      UNTEN
//           XTAL 2   4|    |17   PB5      RECHTS
//           XTAL 1   5|    |16   PB4      OBEN
//  CMD         PD2   6|    |15   PB3      START
//  ACK    	    PD3   7|    |14   PB2
//  ATT         PD4   8|    |13   PB1
//      	    PD5   9|    |12   PB0
//              GND  10|    |11   PD6
//                     +----+


#include <avr/io.h>
#include <util/delay.h>

#define PS2clk 0
#define PS2dat 1
#define PS2cmd 2
#define PS2att 4
#define PS2PORT PORTD
#define PS2IN PIND

#define SET(x,y) (x|=(1<<y))
#define CLR(x,y) (x&=(~(1<<y)))
#define CHK(x,y) (x & (1<<y))
#define TOG(x,y) (x^=(1<<y))

unsigned char PS2byte;
unsigned char PS2data4;
unsigned char PS2data5;
unsigned char i;


void ReadGamepad(void)
{
   SET(PS2PORT,PS2cmd);
   SET(PS2PORT,PS2clk);
   CLR(PS2PORT,PS2att);     // Tanzmatte aktivieren

   _delay_ms(1);
   PS2byte=0x01;            // 1. Byte
   for(i=0;i<8;i++)
   {
      if(CHK(PS2byte,i)) SET(PS2PORT,PS2cmd);
	  else  CLR(PS2PORT,PS2cmd);
      CLR(PS2PORT,PS2clk);
      _delay_ms(1);
      SET(PS2PORT,PS2clk);
   }
   SET(PS2PORT,PS2cmd);

   _delay_ms(1);
   PS2byte=0x42;            // 2. Byte
   for(i=0;i<8;i++)
   {
      if(CHK(PS2byte,i)) SET(PS2PORT,PS2cmd);
	  else  CLR(PS2PORT,PS2cmd);
      CLR(PS2PORT,PS2clk);
      _delay_ms(1);
      SET(PS2PORT,PS2clk);
   }
   SET(PS2PORT,PS2cmd);

   _delay_ms(1);
   for(i=0;i<8;i++)        // 3. Byte
   {
      CLR(PS2PORT,PS2cmd);
      CLR(PS2PORT,PS2clk);
      SET(PS2PORT,PS2clk);
   }
   SET(PS2PORT,PS2cmd);

   _delay_ms(1);
   PS2data4=0;
   for(i=0;i<8;i++)        // 4. Byte
   {
      CLR(PS2PORT,PS2cmd);
      CLR(PS2PORT,PS2clk);
      _delay_ms(1);
	  if(CHK(PS2IN,PS2dat)) SET(PS2data4,i);
      SET(PS2PORT,PS2clk);
   }
   SET(PS2PORT,PS2cmd);

   _delay_ms(1);
   PS2data5=0;
   for(i=0;i<8;i++)        // 5. Byte
   {
      CLR(PS2PORT,PS2cmd);
      CLR(PS2PORT,PS2clk);
      _delay_ms(1);
	  if(CHK(PS2IN,PS2dat)) SET(PS2data5,i);
      SET(PS2PORT,PS2clk);
   }
   SET(PS2PORT,PS2cmd);
   SET(PS2PORT,PS2att);    // Tanzmatte deaktivieren
}

void PORT_Init(void)
{
	DDRB  = 0b11111000;	   // PB3-PB7 als Ausgng konfigurieren
	PORTB = 0b11111000;    // LED1-LED5 ausschalten

    DDRD |= (1<<PS2clk);   // PD0 (clk) als Ausgang konfigurieren
    DDRD &= ~(1<<PS2dat);  // PD1 (dat) als Eingang konfigurieren
    DDRD |= (1<<PS2cmd);   // PD2 (cmd) als Ausgang konfigurieren
    DDRD |= (1<<PS2att);   // PD4 (att) als Ausgang konfigurieren
    PORTD |= 0b00000010;   // Pullupwiderstand fr (dat) aktivieren

    PORTD |= (1<<PS2att);  // Joystick deaktivieren
}


int main(void)
  {
      PORT_Init();

      for(;;)
        {

        ReadGamepad();
        _delay_ms(1);

        if (PS2data4!=0xff)
            {

            if (PS2data4==0xf7)  // Bit 3 auf Low prfen
                {
                PORTB |= 0b11111111;
                PORTB &= 0b11110111; // Start-Button
                }

            if (PS2data4==0xef)  // Bit 4 auf Low prfen
                {
                PORTB |= 0b11111111;
                PORTB &= 0b11101111; // Joystick oben
                }

            if (PS2data4==0xdf)  // Bit 5 auf Low prfen
                {
                PORTB |= 0b11111111;
                PORTB &= 0b11011111; // Joystick rechts
                }

            if (PS2data4==0xbf)  // Bit 6 auf Low prfen
                {
                PORTB |= 0b11111111;
                PORTB &= 0b10111111; // Joystick rechts
                }

                if (PS2data4==0x7f)  // Bit 7 auf Low prfen
                {
                PORTB |= 0b11111111;
                PORTB &= 0b01111111; // Joystick links
                }
            }
        else
            {
                PORTB |= 0b11111111; // keine Taste gedrckt !
            }
        }
  }
